@@ -9,7 +9,8 @@
 #include "end.h"
 #include "files.h"
 #include "libutil.h"
-#include "l_libs.h"
+#include "l-libs.h"
+#include "maybe-bool.h"
 #include "misc.h" // erase_val
 #include "options.h"
 #include "state.h"
@@ -27,17 +28,6 @@
 #define NO_CUSTOM_ALLOCATOR
 #endif
 
-#define CL_RESETSTACK_RETURN(ls, oldtop, retval) \
-    do \
-    {\
-        if (oldtop != lua_gettop(ls)) \
-        { \
-            lua_settop(ls, oldtop); \
-        } \
-        return retval; \
-    } \
-    while (false)
-
 static int  _clua_panic(lua_State *);
 static void _clua_throttle_hook(lua_State *, lua_Debug *);
 #ifndef NO_CUSTOM_ALLOCATOR
@@ -51,7 +41,7 @@ static string _get_persist_file();
 
 CLua::CLua(bool managed)
     : error(), managed_vm(managed), shutting_down(false),
-      throttle_unit_lines(10000),
+      throttle_unit_lines(50000),
       throttle_sleep_ms(0), throttle_sleep_start(2),
       throttle_sleep_end(800), n_throttle_sleeps(0), mixed_call_depth(0),
       lua_call_depth(0), max_mixed_call_depth(8),
@@ -66,8 +56,8 @@ CLua::~CLua()
     // themselves from the listener list when we notify them of a
     // shutdown.
     const vector<lua_shutdown_listener*> slisteners = shutdown_listeners;
-    for (int i = 0, size = slisteners.size(); i < size; ++i)
-        slisteners[i]->shutdown(*this);
+    for (lua_shutdown_listener *listener : slisteners)
+        listener->shutdown(*this);
     shutting_down = true;
     if (_state)
         lua_close(_state);
@@ -142,6 +132,9 @@ void CLua::save_persist()
     // that we know that it hasn't been overwritten by a player version.
     execfile("dlua/persist.lua", true, true);
     callfn("c_save_persist", ">s", &persist);
+    if (Options.no_save)
+        return;
+
     FILE *f;
     const string persistfile = _get_persist_file();
 
@@ -167,6 +160,8 @@ void CLua::save_persist()
 
 void CLua::load_persist()
 {
+    if (Options.no_save)
+        return;
     string persistfile = _get_persist_file();
     if (!file_exists(persistfile))
         return;
@@ -246,6 +241,7 @@ void CLua::init_throttle()
                     LUA_MASKCOUNT, throttle_unit_lines);
         throttle_sleep_ms = 0;
         n_throttle_sleeps = 0;
+        crawl_state.lua_script_killed = false;
     }
 }
 
@@ -309,6 +305,9 @@ int CLua::loadfile(lua_State *ls, const char *filename, bool trusted,
     while (!f.eof())
         script += f.get_line() + "\n";
 
+    if (script[0] == 0x1b)
+        abort();
+
     // prefixing with @ stops lua from adding [string "%s"]
     return luaL_loadbuffer(ls, &script[0], script.length(),
                            ("@" + file).c_str());
@@ -320,18 +319,18 @@ int CLua::execfile(const char *filename, bool trusted, bool die_on_fail,
     if (!force && sourced_files.count(filename))
         return 0;
 
-    sourced_files.insert(filename);
-
     lua_State *ls = state();
     int err = loadfile(ls, filename, trusted || !managed_vm, die_on_fail);
     lua_call_throttle strangler(this);
     if (!err)
         err = lua_pcall(ls, 0, 0, 0);
+    if (!err)
+        sourced_files.insert(filename);
     set_error(err);
     if (die_on_fail && !error.empty())
     {
         end(1, false, "Lua execfile error (%s): %s",
-            filename, dlua.error.c_str());
+            filename, error.c_str());
     }
     return err;
 }
@@ -344,17 +343,15 @@ bool CLua::runhook(const char *hook, const char *params, ...)
     if (!ls)
         return false;
 
-    // Remember top of stack, for debugging porpoises
-    int stack_top = lua_gettop(ls);
+    lua_stack_cleaner clean(ls);
+
     pushglobal(hook);
     if (!lua_istable(ls, -1))
-    {
-        lua_pop(ls, 1);
-        CL_RESETSTACK_RETURN(ls, stack_top, false);
-    }
+        return false;
     for (int i = 1; ; ++i)
     {
-        int currtop = lua_gettop(ls);
+        lua_stack_cleaner clean2(ls);
+
         lua_rawgeti(ls, -1, i);
         if (!lua_isfunction(ls, -1))
         {
@@ -367,10 +364,8 @@ bool CLua::runhook(const char *hook, const char *params, ...)
         va_start(args, params);
         calltopfn(ls, params, args);
         va_end(args);
-
-        lua_settop(ls, currtop);
     }
-    CL_RESETSTACK_RETURN(ls, stack_top, true);
+    return true;
 }
 
 void CLua::fnreturns(const char *format, ...)
@@ -410,7 +405,7 @@ void CLua::vfnreturns(const char *format, va_list args)
             break;
         case 'd':
             if (lua_isnumber(ls, sp))
-                *(va_arg(args, int*)) = luaL_checkint(ls, sp);
+                *(va_arg(args, int*)) = luaL_safe_checkint(ls, sp);
             break;
         case 'b':
             *(va_arg(args, bool *)) = lua_toboolean(ls, sp);
@@ -508,6 +503,8 @@ int CLua::push_args(lua_State *ls, const char *format, va_list args,
 
 int CLua::return_count(lua_State *ls, const char *format)
 {
+    UNUSED(ls);
+
     if (!format)
         return 0;
 
@@ -544,41 +541,34 @@ bool CLua::calltopfn(lua_State *ls, const char *params, va_list args,
     return !err;
 }
 
-static maybe_bool _frombool(bool b)
-{
-    return b ? MB_TRUE : MB_FALSE;
-}
-
 maybe_bool CLua::callmbooleanfn(const char *fn, const char *params,
                                 va_list args)
 {
     error.clear();
     lua_State *ls = state();
     if (!ls)
-        return MB_MAYBE;
+        return maybe_bool::maybe;
 
-    int stacktop = lua_gettop(ls);
+    lua_stack_cleaner clean(ls);
 
     pushglobal(fn);
     if (!lua_isfunction(ls, -1))
-    {
-        lua_pop(ls, 1);
-        CL_RESETSTACK_RETURN(ls, stacktop, MB_MAYBE);
-    }
+        return maybe_bool::maybe;
 
     bool ret = calltopfn(ls, params, args, 1);
     if (!ret)
-        CL_RESETSTACK_RETURN(ls, stacktop, MB_MAYBE);
+        return maybe_bool::maybe;
 
-    maybe_bool r = _frombool(lua_toboolean(ls, -1));
-    CL_RESETSTACK_RETURN(ls, stacktop, r);
+    return lua_toboolean(ls, -1);
 }
 
 maybe_bool CLua::callmbooleanfn(const char *fn, const char *params, ...)
 {
     va_list args;
     va_start(args, params);
-    return callmbooleanfn(fn, params, args);
+    maybe_bool r = callmbooleanfn(fn, params, args);
+    va_end(args);
+    return r;
 }
 
 maybe_bool CLua::callmaybefn(const char *fn, const char *params, va_list args)
@@ -586,45 +576,28 @@ maybe_bool CLua::callmaybefn(const char *fn, const char *params, va_list args)
     error.clear();
     lua_State *ls = state();
     if (!ls)
-        return MB_MAYBE;
+        return maybe_bool::maybe;
 
-    int stacktop = lua_gettop(ls);
+    lua_stack_cleaner clean(ls);
 
     pushglobal(fn);
     if (!lua_isfunction(ls, -1))
-    {
-        lua_pop(ls, 1);
-        CL_RESETSTACK_RETURN(ls, stacktop, MB_MAYBE);
-    }
+        return maybe_bool::maybe;
 
     bool ret = calltopfn(ls, params, args, 1);
-    if (!ret)
-        CL_RESETSTACK_RETURN(ls, stacktop, MB_MAYBE);
+    if (!ret || !lua_isboolean(ls, -1))
+        return maybe_bool::maybe;
 
-    maybe_bool r = lua_isboolean(ls, -1) ? _frombool(lua_toboolean(ls, -1))
-                                         : MB_MAYBE;
-    CL_RESETSTACK_RETURN(ls, stacktop, r);
+    return lua_toboolean(ls, -1);
 }
 
 maybe_bool CLua::callmaybefn(const char *fn, const char *params, ...)
 {
     va_list args;
     va_start(args, params);
-    return callmaybefn(fn, params, args);
-}
-
-static bool _tobool(maybe_bool mb, bool def)
-{
-    switch (mb)
-    {
-    case MB_TRUE:
-        return true;
-    case MB_FALSE:
-        return false;
-    case MB_MAYBE:
-    default:
-        return def;
-    }
+    maybe_bool r = callmaybefn(fn, params, args);
+    va_end(args);
+    return r;
 }
 
 bool CLua::callbooleanfn(bool def, const char *fn, const char *params, ...)
@@ -632,7 +605,8 @@ bool CLua::callbooleanfn(bool def, const char *fn, const char *params, ...)
     va_list args;
     va_start(args, params);
     maybe_bool r = callmbooleanfn(fn, params, args);
-    return _tobool(r, def);
+    va_end(args);
+    return r.to_bool(def);
 }
 
 bool CLua::proc_returns(const char *par) const
@@ -697,6 +671,7 @@ bool CLua::callfn(const char *fn, const char *params, ...)
     va_list args;
     va_list fnret;
     va_start(args, params);
+
     bool ret = calltopfn(ls, params, args, -1, &fnret);
     if (ret)
     {
@@ -761,16 +736,78 @@ void CLua::init_lua()
 
     lua_atpanic(_state, _clua_panic);
 
-    luaopen_base(_state);
-    luaopen_string(_state);
-    luaopen_table(_state);
-    luaopen_math(_state);
+#ifdef CLUA_UNRESTRICTED_LIBS
+    // open all libs -- this is not safe for public servers or releases!
+    // Intended for people writing bots and the like.
+    luaL_openlibs(_state);
+#else
+    // Selectively load some, but not all Lua core libraries.
+    //
+    // In Lua 5.1, these library setup calls are not supposed to be called
+    // directly from C. If the lua version changes, this may need to be changed:
+    // recommended practice is (apparently) checking the lua version's linit.cc
+    // and seeing how that does the full library setup.
+    //
+    // This doesn't seem to *obviously* impact the libraries we use by default,
+    // but some of the libraries we don't use will panic if not called
+    // correctly; since someone writing a bot (for example) might want to
+    // expand this, do things "correctly". The core lua libraries in 5.1 we are
+    // not loading are:
+    //
+    // {LUA_LOADLIBNAME, luaopen_package},    // (require etc)
+    // {LUA_IOLIBNAME, luaopen_io},           //
+    // {LUA_OSLIBNAME, luaopen_os},
+    // {LUA_DBLIBNAME, luaopen_debug},
+    const vector<pair<string, lua_CFunction>> lua_core_libs =
+    {
+        {"", luaopen_base}, // XX: why no name? but this is how linit.cc does it
+        {LUA_TABLIBNAME, luaopen_table},
+        {LUA_STRLIBNAME, luaopen_string},
+        {LUA_MATHLIBNAME, luaopen_math},
+    };
+
+    for (auto l : lua_core_libs)
+    {
+        lua_pushcfunction(_state, l.second);
+        lua_pushstring(_state, l.first.c_str());
+        lua_call(_state, 1, 0);
+    }
+#endif
+
+    lua_pushboolean(_state, managed_vm);
+    setregistry("lua_vm_is_managed");
+
+    lua_pushlightuserdata(_state, this);
+    setregistry("__clua");
+}
+
+static int lua_loadstring(lua_State *ls)
+{
+    const auto lua = luaL_checkstring(ls, 1);
+    if (lua[0] == 0x1b)
+        abort();
+    lua_settop(ls, 0);
+    if (luaL_loadstring(ls, lua))
+    {
+        lua_pushnil(ls);
+        lua_insert(ls, 1);
+    }
+    return lua_gettop(ls);
+}
+
+void CLua::init_libraries()
+{
+    lua_stack_cleaner clean(state());
+
+    lua_pushcfunction(_state, lua_loadstring);
+    lua_setglobal(_state, "loadstring");
+    lua_pushnil(_state);
+    lua_setglobal(_state, "load");
 
     // Open Crawl bindings
     cluaopen_kills(_state);
     cluaopen_you(_state);
     cluaopen_item(_state);
-    cluaopen_food(_state);
     cluaopen_crawl(_state);
     cluaopen_file(_state);
     cluaopen_moninf(_state);
@@ -781,13 +818,15 @@ void CLua::init_lua()
 
     cluaopen_globals(_state);
 
-    load_cmacro();
-    load_chooks();
+    execfile("dlua/macro.lua", true, true);
+
+    // All hook names must be chk_????
+    execstring("chk_startgame = { }", "base");
 
     lua_register(_state, "loadfile", _clua_loadfile);
     lua_register(_state, "dofile", _clua_dofile);
 
-    lua_register(_state, "require", _clua_require);
+    lua_register(_state, "crawl_require", _clua_require);
 
     execfile("dlua/util.lua", true, true);
     execfile("dlua/iter.lua", true, true);
@@ -800,12 +839,6 @@ void CLua::init_lua()
         execfile("dlua/userbase.lua", true, true);
         execfile("dlua/persist.lua", true, true);
     }
-
-    lua_pushboolean(_state, managed_vm);
-    setregistry("lua_vm_is_managed");
-
-    lua_pushlightuserdata(_state, this);
-    setregistry("__clua");
 }
 
 CLua &CLua::get_vm(lua_State *ls)
@@ -826,20 +859,6 @@ bool CLua::is_managed_vm(lua_State *ls)
     return lua_toboolean(ls, -1);
 }
 
-void CLua::load_chooks()
-{
-    // All hook names must be chk_????
-    static const char *c_hooks =
-        "chk_startgame = { }"
-        ;
-    execstring(c_hooks, "base");
-}
-
-void CLua::load_cmacro()
-{
-    execfile("dlua/macro.lua", true, true);
-}
-
 void CLua::add_shutdown_listener(lua_shutdown_listener *listener)
 {
     if (find(shutdown_listeners.begin(), shutdown_listeners.end(), listener)
@@ -855,7 +874,7 @@ void CLua::remove_shutdown_listener(lua_shutdown_listener *listener)
 }
 
 // Can be called from within a debugger to look at the current Lua
-// call stack.  (Borrowed from ToME 3)
+// call stack. (Borrowed from ToME 3)
 void CLua::print_stack()
 {
     struct lua_Debug dbg;
@@ -880,7 +899,7 @@ void CLua::print_stack()
     fprintf(stderr, "\n");
 }
 
-////////////////////////////////////////////////////////////////////////
+// //////////////////////////////////////////////////////////////////////
 // lua_text_pattern
 
 // We could simplify this a great deal by just using lex and yacc, but I
@@ -916,9 +935,9 @@ bool lua_text_pattern::is_lua_pattern(const string &s)
 }
 
 lua_text_pattern::lua_text_pattern(const string &_pattern)
-    : translated(false), isvalid(true), pattern(_pattern), lua_fn_name()
+    : translated(false), isvalid(true), pattern(_pattern),
+      lua_fn_name(new_fn_name())
 {
-    lua_fn_name = new_fn_name();
 }
 
 lua_text_pattern::~lua_text_pattern()
@@ -950,6 +969,17 @@ bool lua_text_pattern::matches(const string &s) const
     return clua.callbooleanfn(false, lua_fn_name.c_str(), "s", s.c_str());
 }
 
+pattern_match lua_text_pattern::match_location(const string &s) const
+{
+    // lua_text_pattern is only used if a special non-regex op is detected (^F
+    // for "armour && ego", for instance), and in those situations, it's
+    // unclear what exactly to use for the matched text here (especially in
+    // more complicated expressions that include things like <<>>, !!, etc).
+    return matches(s)
+        ? pattern_match::succeeded(s)
+        : pattern_match::failed(s);
+}
+
 void lua_text_pattern::pre_pattern(string &pat, string &fn) const
 {
     // Trim trailing spaces
@@ -1037,7 +1067,7 @@ bool lua_text_pattern::translate() const
     return translated;
 }
 
-//////////////////////////////////////////////////////////////////////////
+// ////////////////////////////////////////////////////////////////////////
 
 lua_call_throttle::lua_clua_map lua_call_throttle::lua_map;
 
@@ -1057,6 +1087,7 @@ lua_call_throttle::lua_clua_map lua_call_throttle::lua_map;
 //
 static int _clua_panic(lua_State *ls)
 {
+    UNUSED(ls);
     if (crawl_state.need_save && !crawl_state.saving_game
         && !crawl_state.updating_scores)
     {
@@ -1089,6 +1120,11 @@ static void *_clua_allocator(void *ud, void *ptr, size_t osize, size_t nsize)
 
 static void _clua_throttle_hook(lua_State *ls, lua_Debug *dbg)
 {
+    UNUSED(dbg);
+
+    if (crawl_state.seen_hups)
+        luaL_error(ls, "Aborting clua code on SIGHUP");
+
     CLua *lua = lua_call_throttle::find_clua(ls);
 
     // Co-routines can create a new Lua state; in such cases, we must
@@ -1111,6 +1147,7 @@ static void _clua_throttle_hook(lua_State *ls, lua_Debug *dbg)
         if (lua->n_throttle_sleeps > CLua::MAX_THROTTLE_SLEEPS)
         {
             lua->n_throttle_sleeps = CLua::MAX_THROTTLE_SLEEPS;
+            crawl_state.lua_script_killed = true;
             luaL_error(ls, BUGGY_SCRIPT_ERROR);
         }
     }
@@ -1171,6 +1208,20 @@ static int _clua_guarded_pcall(lua_State *ls)
     return lua_gettop(ls);
 }
 
+// Document clua globals here, as they're bound by the interpreter object
+
+/*** Pre-defined globals.
+ *
+ * *Note:* this is not a real module. All names described here are defined in
+ * the global clua namespace.
+ * @module Globals
+ */
+
+/*** Load the named lua file as a chunk.
+ * @tparam string filename
+ * @return function chunk or nil,error
+ * @function loadfile
+ */
 static int _clua_loadfile(lua_State *ls)
 {
     const char *file = luaL_checkstring(ls, 1);
@@ -1188,6 +1239,13 @@ static int _clua_loadfile(lua_State *ls)
     return 1;
 }
 
+/*** Load and execute the named lua file.
+ * Differs from @{dofile} in that the file is run for its side effects.
+ * If the execution has an error we raise that error and exit.
+ * @tparam string filename
+ * @treturn boolean|nil
+ * @function require
+ */
 static int _clua_require(lua_State *ls)
 {
     const char *file = luaL_checkstring(ls, 1);
@@ -1202,6 +1260,13 @@ static int _clua_require(lua_State *ls)
     return 1;
 }
 
+/*** Load and execute the named luafile, returning the result.
+ * Differs from @{require} in that the file is run for a result. Errors
+ * come back on the lua stack and can be handled by the caller.
+ * @tparam string filename
+ * @return whatever is left on the lua stack by filename
+ * @function dofile
+ */
 static int _clua_dofile(lua_State *ls)
 {
     const char *file = luaL_checkstring(ls, 1);
@@ -1226,7 +1291,7 @@ static string _get_persist_file()
     return Options.filename + ".persist";
 }
 
-/////////////////////////////////////////////////////////////////////
+// ///////////////////////////////////////////////////////////////////
 
 lua_shutdown_listener::~lua_shutdown_listener()
 {
