@@ -17,9 +17,9 @@
 #include "cloud.h"
 #include "directn.h"
 #include "env.h"
-#include "itemprop.h"
+#include "item-prop.h"
 #include "message.h"
-#include "mgen_data.h"
+#include "mgen-data.h"
 #include "mon-death.h"
 #include "mon-place.h"
 #include "ouch.h"
@@ -30,32 +30,30 @@
 
 static void _fuzz_direction(const actor *caster, monster& mon, int pow);
 
-spret_type cast_iood(actor *caster, int pow, bolt *beam, float vx, float vy,
-                     int foe, bool fail)
+spret cast_iood(actor *caster, int pow, bolt *beam, float vx, float vy,
+                     int foe, bool fail, bool needs_tracer)
 {
     const bool is_player = caster->is_player();
-    if (beam && is_player && !player_tracer(ZAP_IOOD, pow, *beam))
-        return SPRET_ABORT;
+    if (beam && is_player && needs_tracer
+             && !player_tracer(ZAP_IOOD, pow, *beam))
+    {
+        return spret::abort;
+    }
 
     fail_check();
 
     int mtarg = !beam ? MHITNOT :
-                beam->target == you.pos() ? MHITYOU : mgrd(beam->target);
+                beam->target == you.pos() ? int{MHITYOU} : env.mgrid(beam->target);
 
     monster *mon = place_monster(mgen_data(MONS_ORB_OF_DESTRUCTION,
                 (is_player) ? BEH_FRIENDLY :
                     ((monster*)caster)->friendly() ? BEH_FRIENDLY : BEH_HOSTILE,
-                caster,
-                0,
-                SPELL_IOOD,
                 coord_def(),
-                mtarg,
-                0,
-                GOD_NO_GOD), true, true);
+                mtarg).set_summoned(caster, 0, SPELL_IOOD), true, true);
     if (!mon)
     {
         mprf(MSGCH_ERROR, "Failed to spawn projectile.");
-        return SPRET_ABORT;
+        return spret::abort;
     }
 
     if (beam)
@@ -68,10 +66,10 @@ spret_type cast_iood(actor *caster, int pow, bolt *beam, float vx, float vy,
             beam->ray.r.start.x - 0.5, beam->ray.r.start.y - 0.5,
             beam->ray.r.dir.x, beam->ray.r.dir.y);
 #endif
-        mon->props["iood_x"].get_float() = beam->ray.r.start.x - 0.5;
-        mon->props["iood_y"].get_float() = beam->ray.r.start.y - 0.5;
-        mon->props["iood_vx"].get_float() = beam->ray.r.dir.x;
-        mon->props["iood_vy"].get_float() = beam->ray.r.dir.y;
+        mon->props[IOOD_X].get_float() = beam->ray.r.start.x - 0.5;
+        mon->props[IOOD_Y].get_float() = beam->ray.r.start.y - 0.5;
+        mon->props[IOOD_VX].get_float() = beam->ray.r.dir.x;
+        mon->props[IOOD_VY].get_float() = beam->ray.r.dir.y;
         _fuzz_direction(caster, *mon, pow);
     }
     else
@@ -79,17 +77,17 @@ spret_type cast_iood(actor *caster, int pow, bolt *beam, float vx, float vy,
         // Multi-orb: spread the orbs a bit, otherwise diagonal ones might
         // fail to leave the cardinal direction: orb A moves -0.4,+0.9 and
         // orb B +0.4,+0.9, both rounded to 0,1.
-        mon->props["iood_x"].get_float() = caster->pos().x + 0.4 * vx;
-        mon->props["iood_y"].get_float() = caster->pos().y + 0.4 * vy;
-        mon->props["iood_vx"].get_float() = vx;
-        mon->props["iood_vy"].get_float() = vy;
+        mon->props[IOOD_X].get_float() = caster->pos().x + 0.4 * vx;
+        mon->props[IOOD_Y].get_float() = caster->pos().y + 0.4 * vy;
+        mon->props[IOOD_VX].get_float() = vx;
+        mon->props[IOOD_VY].get_float() = vy;
     }
 
-    mon->props["iood_kc"].get_byte() = (is_player) ? KC_YOU :
+    mon->props[IOOD_KC].get_byte() = (is_player) ? KC_YOU :
         ((monster*)caster)->friendly() ? KC_FRIENDLY : KC_OTHER;
-    mon->props["iood_pow"].get_short() = pow;
+    mon->props[IOOD_POW].get_short() = pow;
     mon->flags &= ~MF_JUST_SUMMONED;
-    mon->props["iood_caster"].get_string() = caster->as_monster()
+    mon->props[IOOD_CASTER].get_string() = caster->as_monster()
         ? caster->name(DESC_A, true)
         : (caster->is_player()) ? "you" : "";
     mon->summoner = caster->mid;
@@ -97,7 +95,7 @@ spret_type cast_iood(actor *caster, int pow, bolt *beam, float vx, float vy,
     if (caster->is_player() || caster->type == MONS_PLAYER_GHOST
         || caster->type == MONS_PLAYER_ILLUSION)
     {
-        mon->props["iood_flawed"].get_byte() = true;
+        mon->props[IOOD_FLAWED].get_byte() = true;
     }
 
     // Move away from the caster's square.
@@ -117,7 +115,7 @@ spret_type cast_iood(actor *caster, int pow, bolt *beam, float vx, float vy,
             mon->foe = foe;
     }
 
-    return SPRET_SUCCESS;
+    return spret::success;
 }
 
 /**
@@ -143,7 +141,7 @@ static int _burst_iood_target(double iood_angle, int preferred_foe)
         const monster* m = *mi;
         ASSERT(m);
 
-        if (!you.can_see(m) || mons_is_projectile(m))
+        if (!you.can_see(*m) || mons_is_projectile(*m))
             continue;
 
         // is this position at a valid angle?
@@ -168,7 +166,7 @@ static int _burst_iood_target(double iood_angle, int preferred_foe)
             return preferred_foe;
         }
 
-        if (mons_aligned(m, &you) || mons_is_firewood(m))
+        if (mons_aligned(m, &you) || mons_is_firewood(*m))
         {
             dprf("skipping invalid burst target %s (%s)",
                  m->name(DESC_PLAIN).c_str(),
@@ -199,9 +197,9 @@ static int _burst_iood_target(double iood_angle, int preferred_foe)
 void cast_iood_burst(int pow, coord_def target)
 {
     const monster* mons = monster_at(target);
-    const int preferred_foe = mons && you.can_see(mons) ?
-                                                            mons->mindex() :
-                                                            MHITNOT;
+    const int preferred_foe = mons && you.can_see(*mons) ?
+                              mons->mindex() :
+                              MHITNOT;
 
     const int n_orbs = random_range(3, 7);
     dprf("Bursting %d orbs.", n_orbs);
@@ -235,44 +233,36 @@ static void _iood_stop(monster& mon, bool msg = true)
     if (!mon.alive())
         return;
 
-    if (mons_is_boulder(&mon))
-    {
-        // Deduct the energy first - the move they made that just stopped
-        // them was a speed 14 move.
-        mon.lose_energy(EUT_MOVE);
-        mon.del_ench(ENCH_ROLLING,!msg);
-        return;
-    }
-
     if (msg)
-        simple_monster_message(&mon, " dissipates.");
+        simple_monster_message(mon, " dissipates.");
     dprf("iood: dissipating");
-    monster_die(&mon, KILL_DISMISSED, NON_MONSTER);
+    monster_die(mon, KILL_DISMISSED, NON_MONSTER);
 }
 
 static void _fuzz_direction(const actor *caster, monster& mon, int pow)
 {
-    const float x = mon.props["iood_x"];
-    const float y = mon.props["iood_y"];
-    float vx = mon.props["iood_vx"];
-    float vy = mon.props["iood_vy"];
+    const float x = mon.props[IOOD_X];
+    const float y = mon.props[IOOD_Y];
+    float vx = mon.props[IOOD_VX];
+    float vy = mon.props[IOOD_VY];
 
     _normalize(vx, vy);
 
     if (pow < 10)
         pow = 10;
-    const float off = (coinflip() ? -1 : 1) * 0.25;
+    const float off = random_choose(-0.25, 0.25);
     float tan = (random2(31) - 15) * 0.019; // approx from degrees
     tan *= 75.0 / pow;
-    if (caster && caster->inaccuracy())
-        tan *= 2;
+    const int inaccuracy = caster ? caster->inaccuracy() : 0;
+    if (inaccuracy > 0)
+        tan *= 2 * inaccuracy;
 
     // Cast either from left or right hand.
-    mon.props["iood_x"] = x + vy*off;
-    mon.props["iood_y"] = y - vx*off;
+    mon.props[IOOD_X] = x + vy*off;
+    mon.props[IOOD_Y] = y - vx*off;
     // And off the direction a bit.
-    mon.props["iood_vx"] = vx + vy*tan;
-    mon.props["iood_vy"] = vy - vx*tan;
+    mon.props[IOOD_VX] = vx + vy*tan;
+    mon.props[IOOD_VY] = vy - vx*tan;
 }
 
 // Alas, too much differs to reuse beam shield blocks :(
@@ -282,34 +272,29 @@ static bool _iood_shielded(monster& mon, actor &victim)
         return false;
 
     const int to_hit = 15 + (mons_is_projectile(mon.type) ?
-        mon.props["iood_pow"].get_short()/12 : mon.get_hit_dice()/2);
+        mon.props[IOOD_POW].get_short()/12 : mon.get_hit_dice()/2);
     const int con_block = random2(to_hit + victim.shield_block_penalty());
     const int pro_block = victim.shield_bonus();
     dprf("iood shield: pro %d, con %d", pro_block, con_block);
     return pro_block >= con_block;
 }
 
-static bool _boulder_hit(monster& mon, const coord_def &pos)
+dice_def iood_damage(int pow, int dist, bool random)
 {
-    actor *victim = actor_at(pos);
-    if (victim)
+    int flat = 60;
+
+    if (dist < 4)
     {
-        simple_monster_message(&mon, (string(" smashes into ")
-                               + victim->name(DESC_THE) + "!").c_str());
-
-        int dam = victim->apply_ac(roll_dice(3, 20));
-        victim->hurt(&mon, dam, BEAM_MISSILE, KILLED_BY_ROLLING);
+        pow = random ? div_rand_round(pow * dist * 3, 10)
+                     : pow * dist * 3 / 10;
+        flat = flat * dist * 3 / 10;
     }
-
-    noisy(5, pos);
-    return victim && victim->alive() || !mon.alive();
+    return dice_def(9, random ? div_rand_round(flat + pow, 12)
+                              : (flat + pow) / 12 );
 }
 
 static bool _iood_hit(monster& mon, const coord_def &pos, bool big_boom = false)
 {
-    if (mons_is_boulder(&mon))
-        return _boulder_hit(mon, pos);
-
     bolt beam;
     beam.name = "orb of destruction";
     beam.flavour = BEAM_DEVASTATION;
@@ -319,11 +304,11 @@ static bool _iood_hit(monster& mon, const coord_def &pos, bool big_boom = false)
     if (!caster)        // caster is dead/gone, blame the orb itself (as its
         caster = &mon;  // friendliness is correct)
     beam.set_agent(caster);
-    if (mon.props.exists("iood_reflector"))
+    if (mon.props.exists(IOOD_REFLECTOR))
     {
         beam.reflections = 1;
 
-        const mid_t refl_mid = mon.props["iood_reflector"].get_int64();
+        const mid_t refl_mid = mon.props[IOOD_REFLECTOR].get_int64();
 
         if (refl_mid == MID_PLAYER)
             beam.reflector = MID_PLAYER;
@@ -340,15 +325,13 @@ static bool _iood_hit(monster& mon, const coord_def &pos, bool big_boom = false)
     beam.source = pos;
     beam.target = pos;
     beam.hit = AUTOMATIC_HIT;
-    beam.source_name = mon.props["iood_caster"].get_string();
+    beam.source_name = mon.props[IOOD_CASTER].get_string();
+    beam.origin_spell = SPELL_IOOD;
 
-    int pow = mon.props["iood_pow"].get_short();
-    pow = stepdown_value(pow, 30, 30, 200, -1);
-    const int dist = mon.props["iood_distance"].get_int();
+    const int pow = mon.props[IOOD_POW].get_short();
+    const int dist = mon.props[IOOD_DIST].get_int();
     ASSERT(dist >= 0);
-    if (dist < 4)
-        pow = pow * (dist*2+3) / 10;
-    beam.damage = dice_def(9, pow / 4);
+    beam.damage = iood_damage(pow, dist);
 
     if (dist < 3)
         beam.name = "wavering " + beam.name;
@@ -357,7 +340,7 @@ static bool _iood_hit(monster& mon, const coord_def &pos, bool big_boom = false)
     beam.ex_size = 1;
     beam.loudness = 7;
 
-    monster_die(&mon, KILL_DISMISSED, NON_MONSTER);
+    monster_die(mon, KILL_DISMISSED, NON_MONSTER);
 
     if (big_boom)
         beam.explode(true, false);
@@ -370,13 +353,12 @@ static bool _iood_hit(monster& mon, const coord_def &pos, bool big_boom = false)
 // returns true if the orb is gone
 bool iood_act(monster& mon, bool no_trail)
 {
-    bool iood = mons_is_projectile(mon.type);
-    ASSERT(iood || mons_is_boulder(&mon));
+    ASSERT(mons_is_projectile(mon.type));
 
-    float x = mon.props["iood_x"];
-    float y = mon.props["iood_y"];
-    float vx = mon.props["iood_vx"];
-    float vy = mon.props["iood_vy"];
+    float x = mon.props[IOOD_X];
+    float y = mon.props[IOOD_Y];
+    float vx = mon.props[IOOD_VX];
+    float vy = mon.props[IOOD_VY];
 
     dprf("iood_act: pos=(%d,%d) rpos=(%f,%f) v=(%f,%f) foe=%d",
          mon.pos().x, mon.pos().y,
@@ -394,9 +376,8 @@ bool iood_act(monster& mon, bool no_trail)
     // If the target is gone, the orb continues on a ballistic course since
     // picking a new one would require intelligence.
 
-    // Boulders don't home onto their targets.  IOODs can't home in on a
-    // submerged creature.
-    if (iood && foe && !foe->submerged())
+    // IOODs can't home in on a submerged creature.
+    if (foe && !foe->submerged())
     {
         const coord_def target = foe->pos();
         float dx = target.x - x;
@@ -407,10 +388,10 @@ bool iood_act(monster& mon, bool no_trail)
         // Moving diagonally when the orb is just about to hit you
         //      2
         //    ->*1
-        // (from 1 to 2) would be a guaranteed escape.  This may be
+        // (from 1 to 2) would be a guaranteed escape. This may be
         // realistic (strafing!), but since the game has no non-cheesy
         // means of waiting a small fraction of a turn, we don't want it.
-        const int old_t_pos = mon.props["iood_tpos"].get_short();
+        const int old_t_pos = mon.props[IOOD_TPOS].get_short();
         const coord_def rpos(static_cast<int>(round(x)), static_cast<int>(round(y)));
         if (old_t_pos && old_t_pos != (256 * target.x + target.y)
             && (rpos - target).rdist() <= 1
@@ -420,7 +401,7 @@ bool iood_act(monster& mon, bool no_trail)
             vx = dx;
             vy = dy;
         }
-        mon.props["iood_tpos"].get_short() = 256 * target.x + target.y;
+        mon.props[IOOD_TPOS].get_short() = 256 * target.x + target.y;
 
         if (!_in_front(vx, vy, dx, dy, 0.3)) // ~17 degrees
         {
@@ -436,18 +417,20 @@ bool iood_act(monster& mon, bool no_trail)
             dprf("iood: keeping course");
 
         _normalize(vx, vy);
-        mon.props["iood_vx"] = vx;
-        mon.props["iood_vy"] = vy;
+        mon.props[IOOD_VX] = vx;
+        mon.props[IOOD_VY] = vy;
     }
 
 move_again:
+    coord_def starting_pos = (mon.pos() == coord_def()) ?
+                                                coord_def(x, y) : mon.pos();
 
     x += vx;
     y += vy;
 
-    mon.props["iood_x"] = x;
-    mon.props["iood_y"] = y;
-    mon.props["iood_distance"].get_int()++;
+    mon.props[IOOD_X] = x;
+    mon.props[IOOD_Y] = y;
+    mon.props[IOOD_DIST].get_int()++;
 
     const coord_def pos(static_cast<int>(round(x)), static_cast<int>(round(y)));
     if (!in_bounds(pos))
@@ -456,7 +439,7 @@ move_again:
         return true;
     }
 
-    if (iood && mon.props.exists("iood_flawed"))
+    if (mon.props.exists(IOOD_FLAWED))
     {
         const actor *caster = actor_by_mid(mon.summoner);
         if (!caster || caster->pos().origin() ||
@@ -471,56 +454,41 @@ move_again:
         return false;
 
     if (!no_trail)
-    {
-        place_cloud(iood ? CLOUD_MAGIC_TRAIL : CLOUD_DUST_TRAIL, mon.pos(),
-                    2 + random2(3), &mon);
-    }
+        place_cloud(CLOUD_MAGIC_TRAIL, starting_pos, 2 + random2(3), &mon);
 
     actor *victim = actor_at(pos);
     if (cell_is_solid(pos) || victim)
     {
-        if (cell_is_solid(pos))
+        if (cell_is_solid(pos)
+            && you.see_cell(pos)
+            && you.see_cell(starting_pos))
         {
-            const int boulder_noisiness = 5; // don't want this to be big
-            if (you.see_cell(pos) && you.see_cell(mon.pos()))
-            {
-                mprf("%s hits %s", mon.name(DESC_THE, true).c_str(),
-                     feature_description_at(pos, false, DESC_A).c_str());
-                if (!iood)
-                    noisy(boulder_noisiness, pos);
-            }
-            else if (!iood && !silenced(you.pos()))
-                noisy(boulder_noisiness, pos, "You hear a crash.");
-
-            if (!iood) // boulders need to stop now
-            {
-                _iood_stop(mon);
-                return true;
-            }
+            mprf("%s hits %s.", mon.name(DESC_THE, true).c_str(),
+                 feature_description_at(pos, false, DESC_A).c_str());
         }
 
         monster* mons = (victim && victim->is_monster()) ?
             (monster*) victim : 0;
 
-        if (mons && iood && mons_is_projectile(victim->type))
+        if (mons && mons_is_projectile(victim->type))
         {
             // Weak orbs just fizzle instead of exploding.
-            if (mons->props["iood_distance"].get_int() < 2
-                || mon.props["iood_distance"].get_int() < 2)
+            if (mons->props[IOOD_DIST].get_int() < 2
+                || mon.props[IOOD_DIST].get_int() < 2)
             {
-                if (mons->props["iood_distance"].get_int() < 2)
+                if (mons->props[IOOD_DIST].get_int() < 2)
                 {
                     if (you.see_cell(pos))
                         mpr("The orb fizzles.");
-                    monster_die(mons, KILL_DISMISSED, NON_MONSTER);
+                    monster_die(*mons, KILL_DISMISSED, NON_MONSTER);
                 }
 
                 // Return, if the acting orb fizzled.
-                if (mon.props["iood_distance"].get_int() < 2)
+                if (mon.props[IOOD_DIST].get_int() < 2)
                 {
                     if (you.see_cell(pos))
                         mpr("The orb fizzles.");
-                    monster_die(&mon, KILL_DISMISSED, NON_MONSTER);
+                    monster_die(mon, KILL_DISMISSED, NON_MONSTER);
                     return true;
                 }
             }
@@ -531,32 +499,12 @@ move_again:
                 else
                     mpr("You hear a loud magical explosion!");
                 noisy(40, pos);
-                monster_die(mons, KILL_DISMISSED, NON_MONSTER);
+                monster_die(*mons, KILL_DISMISSED, NON_MONSTER);
                 _iood_hit(mon, pos, true);
                 return true;
             }
         }
 
-        if (mons && mons_is_boulder(&mon) && mons_is_boulder(mons))
-        {
-            if (mon.observable())
-            {
-                mpr("The boulders collide with a stupendous crash!");
-                noisy(20, pos);
-            }
-            else
-                noisy(20, pos, "You hear a loud crashing sound!");
-
-            // Remove ROLLING and add DAZED
-            _iood_stop(mon);
-            _iood_stop(*mons);
-            if (!mon.check_clarity(false))
-                mon.add_ench(ENCH_CONFUSION);
-            if (!mons->check_clarity(false))
-                mons->add_ench(ENCH_CONFUSION);
-            return true;
-        }
-
         if (mons && (mons->submerged() || mons->type == MONS_BATTLESPHERE))
         {
             // Try to swap with the submerged creature.
@@ -575,14 +523,13 @@ move_again:
 
         if (victim && _iood_shielded(mon, *victim))
         {
-            item_def *shield = victim->shield();
-            if (!shield || !shield_reflects(*shield))
+            if (!victim->reflection())
             {
                 if (victim->is_player())
                     mprf("You block %s.", mon.name(DESC_THE, true).c_str());
                 else
                 {
-                    simple_monster_message(mons, (" blocks "
+                    simple_monster_message(*mons, (" blocks "
                         + mon.name(DESC_THE, true) + ".").c_str());
                 }
                 victim->shield_block_succeeded(&mon);
@@ -590,36 +537,52 @@ move_again:
                 return true;
             }
 
+            item_def *shield = victim->shield();
             if (victim->is_player())
             {
-                mprf("Your %s reflects %s!",
-                    shield->name(DESC_PLAIN).c_str(),
-                    mon.name(DESC_THE, true).c_str());
-                ident_reflector(shield);
+                if (shield && shield_reflects(*shield))
+                {
+                    mprf("Your %s blocks %s... and reflects it back!",
+                         shield->name(DESC_PLAIN).c_str(),
+                         mon.name(DESC_THE, true).c_str());
+                }
+                else // has reflection property not from shield
+                {
+                    mprf("You block %s... and reflect it back!",
+                         mon.name(DESC_THE, true).c_str());
+                }
             }
             else if (you.see_cell(pos))
             {
                 if (victim->observable())
                 {
-                    mprf("%s reflects %s with %s %s!",
-                        victim->name(DESC_THE, true).c_str(),
-                        mon.name(DESC_THE, true).c_str(),
-                        mon.pronoun(PRONOUN_POSSESSIVE).c_str(),
-                        shield->name(DESC_PLAIN).c_str());
-                    ident_reflector(shield);
+                    if (shield && shield_reflects(*shield))
+                    {
+                        mprf("%s blocks %s with %s %s... and reflects it back!",
+                             victim->name(DESC_THE, true).c_str(),
+                             mon.name(DESC_THE, true).c_str(),
+                             victim->pronoun(PRONOUN_POSSESSIVE).c_str(),
+                             shield->name(DESC_PLAIN).c_str());
+                    }
+                    else
+                    {
+                        mprf("%s reflects off an invisible shield around %s!",
+                             mon.name(DESC_THE, true).c_str(),
+                             victim->name(DESC_THE, true).c_str());
+                    }
                 }
                 else
                 {
-                    mprf("%s bounces off thin air!",
-                        mon.name(DESC_THE, true).c_str());
+                    mprf("%s bounces off of thin air!",
+                         mon.name(DESC_THE, true).c_str());
                 }
             }
             victim->shield_block_succeeded(&mon);
 
             // mid_t is unsigned so won't fit in a plain int
-            mon.props["iood_reflector"] = (int64_t) victim->mid;
-            mon.props["iood_vx"] = vx = -vx;
-            mon.props["iood_vy"] = vy = -vy;
+            mon.props[IOOD_REFLECTOR] = (int64_t) victim->mid;
+            mon.props[IOOD_VX] = vx = -vx;
+            mon.props[IOOD_VY] = vy = -vy;
 
             // Need to get out of the victim's square.
 
@@ -631,21 +594,7 @@ move_again:
         }
 
         if (_iood_hit(mon, pos))
-        {
-            if (!iood)
-                _iood_stop(mon);
             return true;
-        }
-    }
-
-    // Boulders stop at lava/water to prevent unusual behaviour;
-    // skimming across the water like a pebble could be justifiable, but
-    // it raises too many questions.
-    if (!iood && (!feat_has_solid_floor(grd(pos)) || feat_is_water(grd(pos))))
-    {
-        mprf("%s screeches to a halt.", mon.name(DESC_THE, true).c_str());
-        _iood_stop(mon,false);
-        return true;
     }
 
     if (!mon.move_to_pos(pos))
@@ -655,21 +604,20 @@ move_again:
     }
 
     // move_to_pos() just trashed the coords, set them again
-    mon.props["iood_x"] = x;
-    mon.props["iood_y"] = y;
+    mon.props[IOOD_X] = x;
+    mon.props[IOOD_Y] = y;
 
     return false;
 }
 
-// Reduced copy of iood_act to move the orb while the player
-// is off-level. Just goes straight and dissipates instead of
-// hitting anything.
+// Reduced copy of iood_act to move the orb while the player is off-level.
+// Just goes straight and dissipates instead of hitting anything.
 static bool _iood_catchup_move(monster& mon)
 {
-    float x = mon.props["iood_x"];
-    float y = mon.props["iood_y"];
-    float vx = mon.props["iood_vx"];
-    float vy = mon.props["iood_vy"];
+    float x = mon.props[IOOD_X];
+    float y = mon.props[IOOD_Y];
+    float vx = mon.props[IOOD_VX];
+    float vy = mon.props[IOOD_VY];
 
     if (!vx && !vy) // not initialized
     {
@@ -682,9 +630,9 @@ static bool _iood_catchup_move(monster& mon)
     x += vx;
     y += vy;
 
-    mon.props["iood_x"] = x;
-    mon.props["iood_y"] = y;
-    mon.props["iood_distance"].get_int()++;
+    mon.props[IOOD_X] = x;
+    mon.props[IOOD_Y] = y;
+    mon.props[IOOD_DIST].get_int()++;
 
     const coord_def pos(static_cast<int>(round(x)), static_cast<int>(round(y)));
     if (!in_bounds(pos))
@@ -703,13 +651,6 @@ static bool _iood_catchup_move(monster& mon)
         _iood_stop(mon, true);
         return true;
     }
-    // Boulder doesn't travel over water/lava.
-    if (mons_is_boulder(&mon)
-        && (!feat_has_solid_floor(grd(pos)) || feat_is_water(grd(pos))))
-    {
-        _iood_stop(mon, false);
-        return true;
-    }
 
     if (!mon.move_to_pos(pos))
     {
@@ -718,8 +659,8 @@ static bool _iood_catchup_move(monster& mon)
     }
 
     // move_to_pos() just trashed the coords, set them again
-    mon.props["iood_x"] = x;
-    mon.props["iood_y"] = y;
+    mon.props[IOOD_X] = x;
+    mon.props[IOOD_Y] = y;
 
     return false;
 }
@@ -727,12 +668,12 @@ static bool _iood_catchup_move(monster& mon)
 void iood_catchup(monster* mons, int pturns)
 {
     monster& mon = *mons;
-    ASSERT(mon.is_projectile());
+    ASSERT(mons_is_projectile(*mons));
 
     const int moves = pturns * mon.speed / BASELINE_DELAY;
 
     // Handle some cases for IOOD only
-    if (mons_is_projectile(mons))
+    if (mons_is_projectile(*mons))
     {
         if (moves > 50)
         {
@@ -740,7 +681,7 @@ void iood_catchup(monster* mons, int pturns)
             return;
         }
 
-        if (mon.props["iood_kc"].get_byte() == KC_YOU)
+        if (mon.props[IOOD_KC].get_byte() == KC_YOU)
         {
             // Left player's vision.
             _iood_stop(mon, false);
@@ -752,17 +693,3 @@ void iood_catchup(monster* mons, int pturns)
         if (_iood_catchup_move(mon))
             return;
 }
-
-void boulder_start(monster *mon, bolt *beam)
-{
-    mon->add_ench(ENCH_ROLLING);
-    // Work out x/y/vx/vy from beam
-    beam->choose_ray();
-    mon->props["iood_x"].get_float() = beam->ray.r.start.x - 0.5;
-    mon->props["iood_y"].get_float() = beam->ray.r.start.y - 0.5;
-    mon->props["iood_vx"].get_float() = mons_is_fleeing(mon) ?
-        -beam->ray.r.dir.x : beam->ray.r.dir.x;
-    mon->props["iood_vy"].get_float() = mons_is_fleeing(mon) ?
-        -beam->ray.r.dir.y : beam->ray.r.dir.y;
-    iood_act(*mon);
-}
