@@ -7,6 +7,8 @@
 
 #include "mon-tentacle.h"
 
+#include <functional>
+
 #include "act-iter.h"
 #include "coordit.h"
 #include "delay.h"
@@ -14,12 +16,11 @@
 #include "fprop.h"
 #include "libutil.h" // map_find
 #include "losglobal.h"
-#include "mgen_data.h"
-#include "misc.h"
+#include "mgen-data.h"
 #include "mon-death.h"
 #include "mon-place.h"
+#include "nearby-danger.h"
 #include "terrain.h"
-#include "view.h"
 
 const int MAX_KRAKEN_TENTACLE_DIST = 12;
 const int MAX_ACTIVE_KRAKEN_TENTACLES = 4;
@@ -39,6 +40,17 @@ static monster_type _solo_tentacle_to_segment[][2] =
     { MONS_SNAPLASHER_VINE,   MONS_SNAPLASHER_VINE_SEGMENT },
 };
 
+static mgen_data _segment_data(const monster& head, coord_def pos,
+                               monster_type type)
+{
+    mgen_data mg(type, SAME_ATTITUDE((&head)), pos, head.foe, MG_FORCE_PLACE);
+    if (mons_is_zombified(head))
+        mg.base_type = head.type;
+    mg.set_summoned(&head, 0, 0, head.god)
+      .set_col(head.colour);
+    return mg;
+}
+
 bool mons_is_tentacle_head(monster_type mc)
 {
     for (const monster_type (&m)[3] : _head_child_segment)
@@ -96,7 +108,7 @@ bool mons_is_tentacle_or_tentacle_segment(monster_type mc)
 
 monster_type mons_tentacle_parent_type(const monster* mons)
 {
-    const monster_type mc = mons_base_type(mons);
+    const monster_type mc = mons_base_type(*mons);
 
     for (const monster_type (&m)[3] : _head_child_segment)
         if (mc == m[1])
@@ -115,7 +127,7 @@ monster_type mons_tentacle_parent_type(const monster* mons)
 
 monster_type mons_tentacle_child_type(const monster* mons)
 {
-    const monster_type mc = mons_base_type(mons);
+    const monster_type mc = mons_base_type(*mons);
 
     for (const monster_type (&m)[3] : _head_child_segment)
         if (mc == m[0])
@@ -134,12 +146,12 @@ monster_type mons_tentacle_child_type(const monster* mons)
 
 bool monster::is_child_tentacle() const
 {
-    return mons_is_child_tentacle(mons_base_type(this));
+    return mons_is_child_tentacle(mons_base_type(*this));
 }
 
 bool monster::is_child_tentacle_segment() const
 {
-    return mons_is_child_tentacle_segment(mons_base_type(this));
+    return mons_is_child_tentacle_segment(mons_base_type(*this));
 }
 
 bool monster::is_child_monster() const
@@ -149,13 +161,13 @@ bool monster::is_child_monster() const
 
 bool monster::is_child_tentacle_of(const monster* mons) const
 {
-    return mons_base_type(mons) == mons_tentacle_parent_type(this)
+    return mons_base_type(*mons) == mons_tentacle_parent_type(this)
            && tentacle_connect == mons->mid;
 }
 
 bool monster::is_parent_monster_of(const monster* mons) const
 {
-    return mons_base_type(this) == mons_tentacle_parent_type(mons)
+    return mons_base_type(*this) == mons_tentacle_parent_type(mons)
            && mons->tentacle_connect == mid;
 }
 
@@ -163,35 +175,36 @@ bool monster::is_parent_monster_of(const monster* mons) const
 //to the parent monster
 bool mons_tentacle_adjacent(const monster* parent, const monster* child)
 {
-    return mons_is_tentacle_head(mons_base_type(parent))
+    return mons_is_tentacle_head(mons_base_type(*parent))
            && mons_is_tentacle_segment(child->type)
-           && child->props.exists("inwards")
-           && child->props["inwards"].get_int() == (int) parent->mid;
+           && child->props.exists(INWARDS_KEY)
+           && child->props[INWARDS_KEY].get_int() == (int) parent->mid;
 }
 
-bool get_tentacle_head(const monster*& mon)
+const monster& get_tentacle_head(const monster& mon)
 {
+    const monster* m = &mon;
     // For tentacle segments, find the associated tentacle.
-    if (mon->is_child_tentacle_segment())
+    if (m->is_child_tentacle_segment())
     {
-        monster* tentacle = monster_by_mid(mon->tentacle_connect);
+        monster* tentacle = monster_by_mid(m->tentacle_connect);
         if (!tentacle)
-            return false;
+            return *m;
 
-        mon = tentacle;
+        m = tentacle;
     }
 
     // For tentacles, find the associated head.
-    if (mon->is_child_tentacle())
+    if (m->is_child_tentacle())
     {
-        monster* head = monster_by_mid(mon->tentacle_connect);
+        monster* head = monster_by_mid(m->tentacle_connect);
         if (!head)
-            return false;
+            return *m;
 
-        mon = head;
+        m = head;
     }
 
-    return true;
+    return *m;
 }
 
 static void _establish_connection(monster* tentacle,
@@ -207,9 +220,9 @@ static void _establish_connection(monster* tentacle,
     {
         // This is a little awkward now. Oh well. -cao
         if (tentacle != head)
-            tentacle->props["inwards"].get_int() = head->mid;
+            tentacle->props[INWARDS_KEY].get_int() = head->mid;
         else
-            tentacle->props["inwards"].get_int() = MID_NOBODY;
+            tentacle->props[INWARDS_KEY].get_int() = MID_NOBODY;
 
         return;
     }
@@ -217,16 +230,14 @@ static void _establish_connection(monster* tentacle,
     // No base monster case (demonic tentacles)
     if (!monster_at(last->pos))
     {
-        if (monster *connect = create_monster(
-            mgen_data(connector_type, SAME_ATTITUDE(head), head,
-                      0, 0, last->pos, head->foe,
-                      MG_FORCE_PLACE, head->god, MONS_NO_MONSTER, tentacle->mid,
-                      head->colour, PROX_CLOSE_TO_PLAYER)))
+        mgen_data mg = _segment_data(*head, last->pos, connector_type);
+        mg.props[MGEN_TENTACLE_CONNECT] = int(tentacle->mid);
+        if (monster *connect = create_monster(mg))
         {
-            connect->props["inwards"].get_int() = MID_NOBODY;
-            connect->props["outwards"].get_int() = MID_NOBODY;
+            connect->props[INWARDS_KEY].get_int() = MID_NOBODY;
+            connect->props[OUTWARDS_KEY].get_int() = MID_NOBODY;
 
-            if (head->holiness() == MH_UNDEAD)
+            if (head->holiness() & MH_UNDEAD)
                 connect->flags |= MF_FAKE_UNDEAD;
 
             connect->max_hit_points = tentacle->max_hit_points;
@@ -255,28 +266,26 @@ static void _establish_connection(monster* tentacle,
         if (current_mons)
         {
             // Todo verify current monster type
-            current_mons->props["inwards"].get_int() = last_mon->mid;
-            last_mon->props["outwards"].get_int() = current_mons->mid;
+            current_mons->props[INWARDS_KEY].get_int() = last_mon->mid;
+            last_mon->props[OUTWARDS_KEY].get_int() = current_mons->mid;
             break;
         }
 
          // place a connector
-        if (monster *connect = create_monster(
-            mgen_data(connector_type, SAME_ATTITUDE(head), head,
-                      0, 0, current->pos, head->foe,
-                      MG_FORCE_PLACE, head->god, MONS_NO_MONSTER, tentacle->mid,
-                      head->colour, PROX_CLOSE_TO_PLAYER)))
+        mgen_data mg = _segment_data(*head, current->pos, connector_type);
+        mg.props[MGEN_TENTACLE_CONNECT] = int(tentacle->mid);
+        if (monster *connect = create_monster(mg))
         {
             connect->max_hit_points = tentacle->max_hit_points;
             connect->hit_points = tentacle->hit_points;
 
-            connect->props["inwards"].get_int() = last_mon->mid;
-            connect->props["outwards"].get_int() = MID_NOBODY;
+            connect->props[INWARDS_KEY].get_int() = last_mon->mid;
+            connect->props[OUTWARDS_KEY].get_int() = MID_NOBODY;
 
             if (last_mon->type == connector_type)
-                last_mon->props["outwards"].get_int() = connect->mid;
+                last_mon->props[OUTWARDS_KEY].get_int() = connect->mid;
 
-            if (head->holiness() == MH_UNDEAD)
+            if (head->holiness() & MH_UNDEAD)
                 connect->flags |= MF_FAKE_UNDEAD;
 
             if (monster_can_submerge(connect, env.grid(connect->pos())))
@@ -347,7 +356,7 @@ struct tentacle_attack_constraints
                     temp.path_distance += 1;
                 // Can still search through a firewood monster, just at a higher
                 // path cost.
-                else if (mons_at && mons_is_firewood(mons_at)
+                else if (mons_at && mons_is_firewood(*mons_at)
                     && !mons_aligned(base_monster, mons_at))
                 {
                     temp.path_distance += 10;
@@ -566,9 +575,9 @@ static bool _try_tentacle_connect(const coord_def & new_pos,
     if (base_position == new_pos)
     {
         if (tentacle == head)
-            tentacle->props["inwards"].get_int() = MID_NOBODY;
+            tentacle->props[INWARDS_KEY].get_int() = MID_NOBODY;
         else
-            tentacle->props["inwards"].get_int() = head->mid;
+            tentacle->props[INWARDS_KEY].get_int() = head->mid;
         return true;
     }
 
@@ -626,35 +635,19 @@ static void _purge_connectors(monster* tentacle)
             if (hp > 0 && hp < tentacle->hit_points)
                 tentacle->hit_points = hp;
 
-            monster_die(*mi, KILL_MISC, NON_MONSTER, true);
+            monster_die(**mi, KILL_MISC, NON_MONSTER, true);
         }
     }
     ASSERT(tentacle->alive());
 }
 
-struct complicated_sight_check
-{
-    coord_def base_position;
-    bool operator()(monster* mons, actor * test)
-    {
-        return test->visible_to(mons)
-               && cell_see_cell(base_position, test->pos(), LOS_SOLID_SEE);
-    }
-};
-
-static bool _basic_sight_check(monster* mons, actor * test)
-{
-    return mons->can_see(test);
-}
-
-template<typename T>
-static void _collect_foe_positions(monster* mons,
-                                   vector<coord_def> & foe_positions,
-                                   T & sight_check)
+static void _collect_foe_positions(monster *mons,
+                                   vector<coord_def> &foe_positions,
+                                   function<bool(const actor *)> sight_check)
 {
     coord_def foe_pos(-1, -1);
     actor * foe = mons->get_foe();
-    if (foe && sight_check(mons, foe))
+    if (foe && sight_check(foe))
     {
         foe_positions.push_back(mons->get_foe()->pos());
         foe_pos = foe_positions.back();
@@ -662,11 +655,11 @@ static void _collect_foe_positions(monster* mons,
 
     for (monster_iterator mi; mi; ++mi)
     {
-        monster* test = *mi;
-        if (!mons_is_firewood(test)
+        const monster * const test = *mi;
+        if (!mons_is_firewood(*test)
             && !mons_aligned(test, mons)
             && test->pos() != foe_pos
-            && sight_check(mons, test))
+            && sight_check(test))
         {
             foe_positions.push_back(test->pos());
         }
@@ -696,8 +689,8 @@ static int _collect_connection_data(monster* start_monster,
             connection_data[*adj_it].insert(current_count);
         }
 
-        bool basis = current_mon->props.exists("inwards");
-        monster* next = basis ? monster_by_mid(current_mon->props["inwards"].get_int()) : nullptr;
+        bool basis = current_mon->props.exists(INWARDS_KEY);
+        monster* next = basis ? monster_by_mid(current_mon->props[INWARDS_KEY].get_int()) : nullptr;
 
         if (next && next->is_child_tentacle_of(start_monster))
         {
@@ -724,7 +717,7 @@ static int _collect_connection_data(monster* start_monster,
 
 void move_solo_tentacle(monster* tentacle)
 {
-    if (!tentacle || !mons_is_solo_tentacle(mons_base_type(tentacle)))
+    if (!tentacle || !mons_is_solo_tentacle(mons_base_type(*tentacle)))
         return;
 
     vector<coord_def> foe_positions;
@@ -733,16 +726,20 @@ void move_solo_tentacle(monster* tentacle)
     bool severed = tentacle->has_ench(ENCH_SEVERED);
 
     coord_def base_position;
-    if (!tentacle->props.exists("base_position"))
-        tentacle->props["base_position"].get_coord() = tentacle->pos();
+    if (!tentacle->props.exists(BASE_POSITION_KEY))
+        tentacle->props[BASE_POSITION_KEY].get_coord() = tentacle->pos();
 
-    base_position = tentacle->props["base_position"].get_coord();
+    base_position = tentacle->props[BASE_POSITION_KEY].get_coord();
 
     if (!severed)
     {
-        complicated_sight_check base_sight;
-        base_sight.base_position = base_position;
-        _collect_foe_positions(tentacle, foe_positions, base_sight);
+        _collect_foe_positions(tentacle, foe_positions,
+                [tentacle, base_position](const actor *test) -> bool
+                {
+                    return test->visible_to(tentacle)
+                        && cell_see_cell(base_position, test->pos(),
+                                         LOS_SOLID_SEE);
+                });
         attack_foe = !foe_positions.empty();
     }
 
@@ -763,7 +760,7 @@ void move_solo_tentacle(monster* tentacle)
         {
             if (!actor_at(*ai) && tentacle->is_habitable(*ai))
             {
-                tentacle->props["base_position"].get_coord() = *ai;
+                tentacle->props[BASE_POSITION_KEY].get_coord() = *ai;
                 base_position = *ai;
                 break;
             }
@@ -798,7 +795,8 @@ void move_solo_tentacle(monster* tentacle)
         constrictee = actor_by_mid(tentacle->constricting->begin()->first);
 
         // Don't drag things that cannot move
-        if (!constrictee->is_stationary())
+        if (!constrictee->is_stationary()
+            && !constrictee->resists_dislodge("being pulled by " + tentacle->name(DESC_THE)))
         {
             if (retract_found)
             {
@@ -815,7 +813,7 @@ void move_solo_tentacle(monster* tentacle)
                 bool near_tree = false;
                 for (adjacent_iterator ai(constrictee->pos()); ai; ++ai)
                 {
-                    if (feat_is_tree(grd(*ai)))
+                    if (feat_is_tree(env.grid(*ai)))
                     {
                         near_tree = true;
                         break;
@@ -832,7 +830,7 @@ void move_solo_tentacle(monster* tentacle)
                         {
                             for (adjacent_iterator ai2(*ai); ai2; ++ai2)
                             {
-                                if (feat_is_tree(grd(*ai2)))
+                                if (feat_is_tree(env.grid(*ai2)))
                                 {
                                     pull_constrictee = true;
                                     shift_constrictee = true;
@@ -898,13 +896,13 @@ void move_solo_tentacle(monster* tentacle)
     }
 
     // Why do I have to do this move? I don't get it.
-    // specifically, if tentacle isn't registered at its new position on mgrd
+    // specifically, if tentacle isn't registered at its new position on env.mgrid
     // the search fails (sometimes), Don't know why. -cao
     tentacle->move_to_pos(new_pos);
 
     if (pull_constrictee)
     {
-        if (you.can_see(tentacle))
+        if (you.can_see(*tentacle))
         {
             mprf("The vine drags %s backwards!",
                     constrictee->name(DESC_THE).c_str());
@@ -919,7 +917,7 @@ void move_solo_tentacle(monster* tentacle)
         if (constrictee->is_player())
             stop_delay(true);
     }
-    tentacle->clear_far_constrictions();
+    tentacle->clear_invalid_constrictions();
 
     tentacle_connect_constraints connect_costs;
     connect_costs.connection_constraints = &connection_data;
@@ -940,7 +938,7 @@ void move_solo_tentacle(monster* tentacle)
              old_pos.x, old_pos.y, tentacle->mid, visited_count);
 
         // Is it ok to purge the tentacle here?
-        monster_die(tentacle, KILL_MISC, NON_MONSTER, true);
+        monster_die(*tentacle, KILL_MISC, NON_MONSTER, true);
         return;
     }
 
@@ -951,7 +949,9 @@ void move_solo_tentacle(monster* tentacle)
 
 void move_child_tentacles(monster* mons)
 {
-    if (!mons_is_tentacle_head(mons_base_type(mons))
+    ASSERT(mons);
+
+    if (!mons_is_tentacle_head(mons_base_type(*mons))
         || mons->asleep())
     {
         return;
@@ -960,7 +960,11 @@ void move_child_tentacles(monster* mons)
     bool no_foe = false;
 
     vector<coord_def> foe_positions;
-    _collect_foe_positions(mons, foe_positions, _basic_sight_check);
+    _collect_foe_positions(mons, foe_positions,
+                           [mons](const actor *test) -> bool
+                           {
+                               return mons->can_see(*test);
+                           });
 
     //if (!kraken->near_foe())
     if (foe_positions.empty()
@@ -1000,8 +1004,8 @@ void move_child_tentacles(monster* mons)
                 connection_data[*adj_it].insert(current_count);
             }
 
-            bool basis = current_mon->props.exists("inwards");
-            monster* inward = basis ? monster_by_mid(current_mon->props["inwards"].get_int()) : nullptr;
+            bool basis = current_mon->props.exists(INWARDS_KEY);
+            monster* inward = basis ? monster_by_mid(current_mon->props[INWARDS_KEY].get_int()) : nullptr;
 
             if (inward
                 && (inward->is_child_tentacle_of(tentacle)
@@ -1028,7 +1032,7 @@ void move_child_tentacles(monster* mons)
             // Drop the tentacle if no enemies are in sight and it is
             // adjacent to the main body. This is to prevent players from
             // just sniping tentacles while outside the kraken's fov.
-            monster_die(tentacle, KILL_MISC, NON_MONSTER, true);
+            monster_die(*tentacle, KILL_MISC, NON_MONSTER, true);
             continue;
         }
 
@@ -1049,8 +1053,9 @@ void move_child_tentacles(monster* mons)
         if (tentacle->is_constricting() && retract_found)
         {
             constrictee = actor_by_mid(tentacle->constricting->begin()->first);
-            if (feat_has_solid_floor(grd(old_pos))
-                && constrictee->is_habitable(old_pos))
+            if (feat_has_solid_floor(env.grid(old_pos))
+                && constrictee->is_habitable(old_pos)
+                && !constrictee->resists_dislodge("being pulled by " + tentacle->name(DESC_THE)))
             {
                 pull_constrictee = true;
             }
@@ -1088,12 +1093,12 @@ void move_child_tentacles(monster* mons)
 
         // Why do I have to do this move? I don't get it.
         // specifically, if tentacle isn't registered at its new position on
-        // mgrd the search fails (sometimes), Don't know why. -cao
+        // env.mgrid the search fails (sometimes), Don't know why. -cao
         tentacle->move_to_pos(new_pos);
 
         if (pull_constrictee)
         {
-            if (you.can_see(tentacle))
+            if (you.can_see(*tentacle))
             {
                 mprf("The tentacle pulls %s backwards!",
                      constrictee->name(DESC_THE).c_str());
@@ -1108,7 +1113,7 @@ void move_child_tentacles(monster* mons)
             if (constrictee->is_player())
                 stop_delay(true);
         }
-        tentacle->clear_far_constrictions();
+        tentacle->clear_invalid_constrictions();
 
         connect_costs.connection_constraints = &connection_data;
         connect_costs.base_monster = tentacle;
@@ -1121,8 +1126,8 @@ void move_child_tentacles(monster* mons)
         // in some way. Should look into this more at some point -cao
         if (!connected)
         {
-            mgrd(tentacle->pos()) = tentacle->mindex();
-            monster_die(tentacle, KILL_MISC, NON_MONSTER, true);
+            env.mgrid(tentacle->pos()) = tentacle->mindex();
+            monster_die(*tentacle, KILL_MISC, NON_MONSTER, true);
 
             continue;
         }
@@ -1145,9 +1150,9 @@ static monster* _mons_get_parent_monster(monster* mons)
 
 // When given either a tentacle end or segment, kills the end and all segments
 // of that tentacle.
-int destroy_tentacle(monster* mons)
+bool destroy_tentacle(monster* mons)
 {
-    int seen = 0;
+    bool any = false;
 
     monster* head = mons_is_tentacle_segment(mons->type)
             ? _mons_get_parent_monster(mons) : mons;
@@ -1155,7 +1160,7 @@ int destroy_tentacle(monster* mons)
     //If we tried to find the head, but failed (probably because it is already
     //dead), cancel trying to kill this tentacle
     if (head == nullptr)
-        return 0;
+        return false;
 
     // Some issue with using monster_die leading to DEAD_MONSTER
     // or w/e. Using hurt seems to cause more problems though.
@@ -1163,46 +1168,42 @@ int destroy_tentacle(monster* mons)
     {
         if (mi->is_child_tentacle_of(head))
         {
-            if (mons_near(*mi))
-                seen++;
+            any = true;
             //mi->hurt(*mi, INSTANT_DEATH);
-            monster_die(*mi, KILL_MISC, NON_MONSTER, true);
+            monster_die(**mi, KILL_MISC, NON_MONSTER, true);
         }
     }
 
     if (mons != head)
     {
-        if (mons_near(head))
-                seen++;
-
-        monster_die(head, KILL_MISC, NON_MONSTER, true);
+        any = true;
+        monster_die(*head, KILL_MISC, NON_MONSTER, true);
     }
 
-    return seen;
+    return any;
 }
 
-int destroy_tentacles(monster* head)
+bool destroy_tentacles(monster* head)
 {
-    int seen = 0;
+    bool any = false;
     for (monster_iterator mi; mi; ++mi)
     {
         if (mi->is_child_tentacle_of(head))
         {
-            if (destroy_tentacle(*mi))
-                seen++;
+            any |= destroy_tentacle(*mi);
             if (!mi->is_child_tentacle_segment())
             {
-                monster_die(mi->as_monster(), KILL_MISC, NON_MONSTER, true);
-                seen++;
+                monster_die(*mi->as_monster(), KILL_MISC, NON_MONSTER, true);
+                any = true;
             }
         }
     }
-    return seen;
+    return any;
 }
 
 static int _max_tentacles(const monster* mon)
 {
-    if (mons_base_type(mon) == MONS_KRAKEN)
+    if (mons_base_type(*mon) == MONS_KRAKEN)
         return MAX_ACTIVE_KRAKEN_TENTACLES;
     else if (mon->type == MONS_TENTACLED_STARSPAWN)
         return MAX_ACTIVE_STARSPAWN_TENTACLES;
@@ -1238,7 +1239,7 @@ void mons_create_tentacles(monster* head)
     // unoccupied.
     for (adjacent_iterator adj_it(head->pos()); adj_it; ++adj_it)
     {
-        if (monster_habitable_grid(tent_type, grd(*adj_it))
+        if (monster_habitable_grid(tent_type, env.grid(*adj_it))
             && !actor_at(*adj_it))
         {
             adj_squares.push_back(*adj_it);
@@ -1254,24 +1255,21 @@ void mons_create_tentacles(monster* head)
 
     for (int i = 0 ; i < possible_count; ++i)
     {
-        if (monster *tentacle = create_monster(
-            mgen_data(tent_type, SAME_ATTITUDE(head), head,
-                        0, 0, adj_squares[i], head->foe,
-                        MG_FORCE_PLACE, head->god, MONS_NO_MONSTER,
-                        head->mid, head->colour,
-                        PROX_CLOSE_TO_PLAYER)))
+        mgen_data mg = _segment_data(*head, adj_squares[i], tent_type);
+        mg.props[MGEN_TENTACLE_CONNECT] = int(head->mid);
+        if (monster *tentacle = create_monster(mg))
         {
-            if (you.can_see(tentacle))
+            if (you.can_see(*tentacle))
                 visible_count++;
 
-            tentacle->props["inwards"].get_int() = head->mid;
+            tentacle->props[INWARDS_KEY].get_int() = head->mid;
 
-            if (head->holiness() == MH_UNDEAD)
+            if (head->holiness() & MH_UNDEAD)
                 tentacle->flags |= MF_FAKE_UNDEAD;
         }
     }
 
-    if (mons_base_type(head) == MONS_KRAKEN)
+    if (mons_base_type(*head) == MONS_KRAKEN)
     {
         if (visible_count == 1)
             mpr("A tentacle rises from the water!");
