@@ -2,16 +2,17 @@
  * @file
  * @brief Travel stuff
 **/
-#ifndef TRAVEL_H
-#define TRAVEL_H
+#pragma once
 
 #include <cstdio>
 #include <map>
 #include <string>
 #include <vector>
 
+#include "command-type.h"
+#include "daction-type.h"
 #include "exclude.h"
-#include "travel_defs.h"
+#include "travel-defs.h"
 
 class reader;
 class writer;
@@ -45,14 +46,15 @@ enum run_mode_type
     RMODE_NOT_RUNNING    = 0,  // must remain equal to 0
     RMODE_CONTINUE,
     RMODE_START,
-    RMODE_REST_DURATION = 100,
+    RMODE_WAIT_DURATION = 100,
+    RMODE_REST_DURATION = 9999999, // just rest until fully healed
     RMODE_CONNECTIVITY,        // Pathfinding connectivity check, not running.
 };
 
 /* ***********************************************************************
  * Initialises the travel subsystem.
  */
-void stop_running();
+void stop_running(bool clear_delays = true);
 void travel_init_load_level();
 void travel_init_new_level();
 
@@ -62,10 +64,16 @@ bool is_resting();
 void explore_pickup_event(int did_pickup, int tried_pickup);
 bool feat_is_traversable_now(dungeon_feature_type feat, bool try_fallback = false);
 bool feat_is_traversable(dungeon_feature_type feat, bool try_fallback = false);
+bool is_travelsafe_square(const coord_def& c,
+                                  bool ignore_hostile = false,
+                                  bool ignore_danger = false,
+                                  bool try_fallback = false);
+
 bool is_known_branch_id(branch_type branch);
 bool is_unknown_stair(const coord_def &p);
+bool is_unknown_transporter(const coord_def &p);
 
-void find_travel_pos(const coord_def& youpos, int *move_x, int *move_y,
+void fill_travel_point_distance(const coord_def& youpos,
                      vector<coord_def>* coords = nullptr);
 
 bool is_stair_exclusion(const coord_def &p);
@@ -93,7 +101,8 @@ void start_travel(const coord_def& p);
 
 command_type travel();
 
-void prevent_travel_to(const string &dungeon_feature_name);
+int prevent_travel_to(const string &dungeon_feature_name);
+void reset_travel_terrain();
 
 // Sort dungeon features as appropriate.
 int level_distance(level_id first, level_id second);
@@ -103,8 +112,6 @@ bool branch_entered(branch_type branch);
 bool can_travel_to(const level_id &lid);
 bool can_travel_interlevel();
 
-bool travel_kill_monster(monster_type mons);
-
 enum translevel_prompt_flags
 {
     TPF_NO_FLAGS          = 0,
@@ -147,53 +154,52 @@ extern travel_distance_grid_t travel_point_distance;
 
 enum explore_stop_type
 {
-    ES_NONE                      = 0x0000,
+    ES_NONE                      = 0x00000,
 
     // Explored into view of an item that is NOT eligible for autopickup.
-    ES_ITEM                      = 0x0001,
+    ES_ITEM                      = 0x00001,
 
     // Picked up an item during greedy explore; will stop for anything
     // that's not explicitly ignored and that is not gold.
-    ES_GREEDY_PICKUP             = 0x0002,
+    ES_GREEDY_PICKUP             = 0x00002,
 
     // Stop when picking up gold with greedy explore.
-    ES_GREEDY_PICKUP_GOLD        = 0x0004,
+    ES_GREEDY_PICKUP_GOLD        = 0x00004,
 
     // Picked up an item during greedy explore, ignoring items that were
     // thrown by the PC, and items that the player already has one of in
     // inventory, or a bunch of other conditions (see
     // _interesting_explore_pickup in items.cc)
-    ES_GREEDY_PICKUP_SMART       = 0x0008,
+    ES_GREEDY_PICKUP_SMART       = 0x00008,
 
     // Greedy-picked up an item previously thrown by the PC.
-    ES_GREEDY_PICKUP_THROWN      = 0x0010,
+    ES_GREEDY_PICKUP_THROWN      = 0x00010,
     ES_GREEDY_PICKUP_MASK        = (ES_GREEDY_PICKUP
                                     | ES_GREEDY_PICKUP_GOLD
                                     | ES_GREEDY_PICKUP_SMART
                                     | ES_GREEDY_PICKUP_THROWN),
 
     // Explored into view of an item eligible for autopickup.
-    ES_GREEDY_ITEM               = 0x0020,
+    ES_GREEDY_ITEM               = 0x00020,
 
     // Stepped onto a stack of items that was previously unknown to
     // the player (for instance, when stepping onto the heap of items
     // of a freshly killed monster).
-    ES_GREEDY_VISITED_ITEM_STACK = 0x0040,
+    ES_GREEDY_VISITED_ITEM_STACK = 0x00040,
 
     // Explored into view of a stair, shop, altar, portal, glowing
-    // item, artefact, or branch entrance.
-    ES_STAIR                     = 0x0080,
-    ES_SHOP                      = 0x0100,
-    ES_ALTAR                     = 0x0200,
-    ES_PORTAL                    = 0x0400,
-    ES_GLOWING_ITEM              = 0x0800,
-    ES_ARTEFACT                  = 0x1000,
-    ES_RUNE                      = 0x2000,
-    ES_BRANCH                    = 0x4000,
-    ES_RUNED_DOOR                = 0x8000,
-
-    // Explored into view of an item which can be sacrificed
-    ES_GREEDY_SACRIFICEABLE      = 0x10000,
+    // item, artefact, or branch entrance.... etc.
+    ES_STAIR                     = 0x00080,
+    ES_SHOP                      = 0x00100,
+    ES_ALTAR                     = 0x00200,
+    ES_PORTAL                    = 0x00400,
+    ES_GLOWING_ITEM              = 0x00800,
+    ES_ARTEFACT                  = 0x01000,
+    ES_RUNE                      = 0x02000,
+    ES_BRANCH                    = 0x04000,
+    ES_RUNED_DOOR                = 0x08000,
+    ES_TRANSPORTER               = 0x10000,
+    ES_RUNELIGHT                 = 0x20000,
 };
 
 ////////////////////////////////////////////////////////////////////////////
@@ -209,8 +215,8 @@ public:
     void found_feature(const coord_def &pos, dungeon_feature_type grid);
     void found_item(const coord_def &pos, const item_def &item);
 
-    // Reports discoveries and prompts the player to stop (if necessary).
-    bool prompt_stop() const;
+    // Reports discoveries and stops exploration (if necessary).
+    bool stop_explore() const;
 
 private:
     template <class Z> struct named_thing
@@ -228,7 +234,6 @@ private:
     };
 
     bool can_autopickup;
-    bool sacrifice;
     int es_flags;
     const LevelStashes *current_level;
     vector< named_thing<item_def> > items;
@@ -237,6 +242,8 @@ private:
     vector< named_thing<int> > shops;
     vector< named_thing<int> > altars;
     vector< named_thing<int> > runed_doors;
+    vector< named_thing<int> > transporters;
+    vector< named_thing<int> > runelights;
 
     vector<string> marker_msgs;
     vector<string> marked_feats;
@@ -260,6 +267,7 @@ public:
     {
         PHYSICAL,
         PLACEHOLDER,
+        MAPPED,
     };
 
 public:
@@ -267,7 +275,7 @@ public:
     dungeon_feature_type grid; // Grid feature of the stair.
     level_pos destination;  // The level and the position on the level this
                             // stair leads to. This may be a guess.
-    int       distance;     // The distance traveled to reach this stair.
+    int       distance;     // The distance travelled to reach this stair.
     bool      guessed_pos;  // true if we're not sure that 'destination' is
                             // correct.
     stair_type type;
@@ -285,7 +293,35 @@ public:
 
     string describe() const;
 
-    bool can_travel() const { return type == PHYSICAL; }
+    bool can_travel() const { return type != PLACEHOLDER; }
+};
+
+struct transporter_info
+{
+public:
+    enum transporter_type
+    {
+        PHYSICAL,
+        MAPPED,
+    };
+
+public:
+    coord_def position;     // Position of transporter
+    coord_def destination;  // The position on the level this transporter leads
+                            // to.
+    transporter_type type;
+
+    transporter_info(const coord_def &p, const coord_def &d,
+                     transporter_type t)
+        : position(p), destination(d), type(t) {
+    }
+
+    transporter_info()
+        : position(-1, -1), destination(), type(PHYSICAL) {
+    }
+
+    void save(writer&) const;
+    void load(reader&);
 };
 
 // Information on a level that interlevel travel needs.
@@ -304,11 +340,18 @@ struct LevelInfo
         return stairs;
     }
 
-    stair_info *get_stair(int x, int y);
+    vector<transporter_info> &get_transporters()
+    {
+        return transporters;
+    }
+
     stair_info *get_stair(const coord_def &pos);
+    transporter_info *get_transporter(const coord_def &pos);
     bool empty() const;
     bool know_stair(const coord_def &pos) const;
+    bool know_transporter(const coord_def &pos) const;
     int get_stair_index(const coord_def &pos) const;
+    int get_transporter_index(const coord_def &pos) const;
 
     void clear_distances();
     void set_level_excludes();
@@ -329,6 +372,7 @@ struct LevelInfo
     // Updates/creates a StairInfo for the stair at stairpos in grid coordinates
     void update_stair(const coord_def& stairpos, const level_pos &p,
                       bool guess = false);
+    void update_transporter(const coord_def& transpos, const coord_def &dest);
 
     // Clears all stair info for stairs matching this grid type.
     void clear_stairs(dungeon_feature_type grid);
@@ -343,8 +387,10 @@ private:
     // Gets a list of coordinates of all player-known stairs on the current
     // level.
     static void get_stairs(vector<coord_def> &stairs);
+    static void get_transporters(vector<coord_def> &transporters);
 
     void correct_stair_list(const vector<coord_def> &s);
+    void correct_transporter_list(const vector<coord_def> &s);
     void update_stair_distances();
     void sync_all_branch_stairs();
     void sync_branch_stairs(const stair_info *si);
@@ -353,6 +399,7 @@ private:
 
 private:
     vector<stair_info> stairs;
+    vector<transporter_info> transporters;
 
     // Squares that are not safe to travel to.
     exclude_set excludes;
@@ -367,7 +414,7 @@ private:
     void resize_stair_distances();
 };
 
-const int TRAVEL_WAYPOINT_COUNT = 10;
+const int TRAVEL_WAYPOINT_COUNT = 100;
 // Tracks all levels that the player has seen.
 class TravelCache
 {
@@ -392,7 +439,8 @@ public:
         levels.erase(lev);
     }
 
-    bool know_stair(const coord_def &c) const;
+    bool know_stair(const coord_def &c);
+    bool know_transporter(const coord_def &c);
     bool know_level(const level_id &lev) const
     {
         return levels.count(lev);
@@ -409,13 +457,18 @@ public:
     void set_level_excludes();
 
     void add_waypoint(int x = -1, int y = -1);
+    void set_waypoint(int waynum, int x, int y);
     void delete_waypoint();
     uint8_t is_waypoint(const level_pos &lp) const;
+    vector<string> get_waypoint_descs() const;
     void list_waypoints() const;
+    void flush_invalid_waypoints();
     void update_waypoints() const;
+    bool is_valid_waypoint(int waynum) const;
 
     void update_excludes();
     void update();
+    void update_transporter(const coord_def &c);
 
     void save(writer&) const;
     void load(reader&, int minorVersion);
@@ -428,6 +481,7 @@ public:
     void clear_daction_counter(daction_type c);
 
 private:
+    void update_stone_stair(const coord_def &c);
     void fixup_levels();
 
 private:
@@ -458,21 +512,13 @@ public:
     // position) and destination.
     void set_src_dst(const coord_def &src, const coord_def &dst);
 
-    // Request that the point distance array be annotated with magic numbers for
-    // excludes and waypoints.
-    void set_annotate_map(bool annotate);
-
-    // Sets the travel_distance_grid_t to use instead of travel_point_distance.
-    void set_distance_grid(travel_distance_grid_t distgrid);
-
-    // Set feature vector to use; if non-nullptr, also sets annotate_map to true.
+    // Set feature vector to use; if non-nullptr, also sets annotate_map to
+    // true.
     void set_feature_vector(vector<coord_def> *features);
 
     // Extract features without pathfinding
     void get_features();
 
-    const set<coord_def> get_unreachables() const;
-
     // The next square to go to to move towards the travel destination. Return
     // value is undefined if pathfind was not called with RMODE_TRAVEL.
     const coord_def travel_move() const;
@@ -481,20 +527,14 @@ public:
     // pathfind was not called with RMODE_EXPLORE or RMODE_EXPLORE_GREEDY.
     const coord_def explore_target() const;
 
-    // Nearest greed-inducing square. Return value is undefined if
-    // pathfind was not called with RMODE_EXPLORE_GREEDY.
-    const coord_def greedy_square() const;
-
-    // Nearest unexplored territory. Return value is undefined if
-    // pathfind was not called with RMODE_EXPLORE or
-    // RMODE_EXPLORE_GREEDY.
-    const coord_def unexplored_square() const;
-
     inline void set_ignore_danger()
     {
         ignore_danger = true;
     }
 
+    // Determine if the level is fully explored, when called after pathfind().
+    int explore_status();
+
 protected:
     bool is_greed_inducing_square(const coord_def &c) const;
     bool path_examine_point(const coord_def &c);
@@ -545,9 +585,6 @@ protected:
     // Can we autopickup?
     bool autopickup;
 
-    // Does god wants sacrifices?
-    bool sacrifice;
-
     // Targets for explore and greedy explore.
     coord_def unexplored_place, greedy_place;
 
@@ -567,14 +604,10 @@ protected:
 
     travel_distance_col *point_distance;
 
-    // How many points are we currently considering? We start off with just one
-    // point, and spread outwards like a flood-filler.
-    int points;
-
     // How many points we'll consider next iteration.
     int next_iter_points;
 
-    // How far we've traveled from (start_x, start_y), in moves (a diagonal move
+    // How far we've travelled from (start_x, start_y), in moves (a diagonal move
     // is no longer than an orthogonal move).
     int traveled_distance;
 
@@ -594,9 +627,10 @@ extern TravelCache travel_cache;
 
 void do_interlevel_travel();
 
-// Travel from a mouse click.  Take one step if not safe.  Attack if adjacent.
+// Travel from a mouse click. Take one step if not safe. Attack if adjacent.
 // If force is true, then the player will attack empty squares/open doors.
 #ifdef USE_TILE
+bool click_travel_safe(const coord_def &gc);
 int click_travel(const coord_def &gc, bool force);
 #endif
 
@@ -606,4 +640,4 @@ void clear_level_target();
 void clear_travel_trail();
 int travel_trail_index(const coord_def& gc);
 
-#endif // TRAVEL_H
+bool stairs_destination_is_excluded(const stair_info &si);
